@@ -147,9 +147,10 @@ autogen_pkg()
     cat > configure.ac <<EOF
 dnl -- WARNING: generated by $0, changes will be lost
 AC_PREREQ(2.52)
-AC_INIT(autogen-pkg.sh)
+AC_INIT($package, $version)
+AC_CONFIG_SRCDIR(autogen-pkg.sh)
 AM_CONFIG_HEADER(config.h)
-AM_INIT_AUTOMAKE($package, $version)
+AM_INIT_AUTOMAKE(-Wno-portability)
 
 AC_SUBST(VERSION,$version)
 
