@@ -56,6 +56,14 @@ SCM scm_str2string (const char *src);
 #endif
 
 char* scm_to_locale_string_dynwind (SCM s);
+void scm_dynwind_guile_v__p_p (void* (*dynwind)(void*(*)(void*), void*), void *func,
+                               void *arg1, void *arg2);
+void scm_dynwind_guile_v__p_u_p_p (void* (*dynwind)(void*(*)(void*), void*), void *func,
+                                   void *arg1, unsigned int arg2, void *arg3,
+                                   void *arg4);
+void scm_dynwind_guile_v__p_u_c_p (void* (*dynwind)(void*(*)(void*), void*), void *func,
+                                   void *arg1, unsigned int arg2, const void *arg3,
+                                   void *arg4);
 
 G_END_DECLS
 
