@@ -20,8 +20,9 @@
 
 ;;; Commentary:
 ;;
-;;This module implements some procedures useful to modules that use
-;;g-wrapped libraries.
+;; @c
+;;
+;; Support routines for automatically-generated scheme G-Wrap modules.
 ;;
 ;;; Code:
 
@@ -43,6 +44,17 @@
 @code{use-modules}.")
 
 (define (export-all-lazy! symbols)
+  "Export the @var{symbols} from the current module.
+
+Most generic functions and classes that G-Wrap defines are bound lazily,
+as needed in evaluation. This is done by placing module binder
+procedures on the generated modules. However, if we export all symbols
+by name, this will force the binding eagerly for all values, which is
+slow.
+
+This procedure exports all bindings named in @var{symbols} that are
+already bound in the current module, and then installs a module binder
+procedure on the public interface, which allows lazy binding to work."
   (define (symbol-in? s exp)
     (if (pair? exp)
         (let lp ((exp exp))
@@ -62,5 +74,18 @@
       ;; all classes
       (module-export! mod symbols))
      (else
-      ;; We have a sensible module-make-local-var!; allow lazy bindings.
-      (module-use! (module-public-interface mod) mod)))))
+      ;; We have a sensible module-make-local-var!; export the
+      ;; already-bound variables, and install a module binder in the
+      ;; interface to lazily bind the rest.
+      (let ((obarray (module-obarray mod)))
+        (module-export! mod
+                        (filter
+                         (lambda (s)
+                           (hashq-ref obarray s))
+                         symbols)))
+      (set-module-binder!
+       (module-public-interface mod)
+       (lambda (interface sym define?)
+         (let ((var (module-local-variable mod sym)))
+           (if var (module-add! interface sym var))
+           var)))))))
