@@ -96,8 +96,9 @@ guile_gnome_scm_to_canvas_points (SCM scm)
   if (scm_vector_p (scm) == SCM_BOOL_T)
     {
       int length = SCM_VECTOR_LENGTH (scm);
+      int i;
       points = gnome_canvas_points_new (length);
-      for (int i = 0; i < length; i++)
+      for (i = 0; i < length; i++)
 	{
 	  SCM s = SCM_VECTOR_REF (scm, i);
 	  /* Just return NULL? */
@@ -153,9 +154,7 @@ _wrap_gnome_canvas_points_new (SCM scm)
   SCM_ASSERT_TYPE (scm_vector_p (scm) == SCM_BOOL_T, scm, SCM_ARG1, FUNC_NAME,
 		   "points");
   points = guile_gnome_scm_to_canvas_points (scm);
-  spoints = scm_c_make_gvalue (GNOME_TYPE_CANVAS_POINTS);
-  g_value_set_boxed_take_ownership ((GValue*) SCM_SMOB_DATA (spoints), points);
-  return spoints;
+  return scm_c_gvalue_new_take_boxed (GNOME_TYPE_CANVAS_POINTS, points);
 }
 #endif
 
